import fs from 'node:fs';
import path from 'node:path';
import { resolve } from 'import-meta-resolve';
import chalk from 'chalk';
import tsj from 'ts-json-schema-generator';
import { printDebug } from '../utils.js';
import { TYPES_FILE_NAME } from '../constants.js';
import { parse } from './config.service.js';
import { shouldQuote } from '@nangohq/nango-yaml';
import { NANGO_VERSION } from '../version.js';
/**
 * Load nango.yaml and generate model.ts
 */
export function loadYamlAndGenerate({ fullPath, debug = false }) {
    if (debug) {
        printDebug(`Generating ${TYPES_FILE_NAME} file`);
    }
    const fp = path.resolve(fullPath, TYPES_FILE_NAME);
    if (!fs.existsSync(fp)) {
        if (debug) {
            printDebug('First compilation');
        }
    }
    else {
        if (debug) {
            printDebug(`File already exists, replacing`);
        }
    }
    const { success, error, response } = parse(fullPath, debug);
    if (!success || !response?.parsed) {
        console.log(chalk.red(error?.message));
        return { success: false, error: null, response: null };
    }
    const modelTs = buildModelsTS({ parsed: response.parsed });
    fs.writeFileSync(fp, modelTs);
    if (debug) {
        printDebug(`${TYPES_FILE_NAME} generated`);
    }
    // --- Additional exports
    generateAdditionalExports({ parsed: response.parsed, fullPath, debug });
    return { success: true, error: null, response: response.parsed };
}
export function loadSchemaJson({ fullPath }) {
    const filePath = path.join(fullPath, '.nango', 'schema.json');
    try {
        return JSON.parse(fs.readFileSync(filePath).toString());
    }
    catch (error) {
        console.error(chalk.red(`Error loading ${filePath}`), error);
        return null;
    }
}
/**
 * Build models.ts
 */
export function buildModelsTS({ parsed }) {
    return `// ---------------------------
// This file was generated by Nango (v${NANGO_VERSION})
// It's recommended to version this file
// https://nango.dev
// ---------------------------

// ------ Models
${generateInterfaces({ parsed }).join('\n\n')}
// ------ /Models

// ------ SDK
${generateSDKTypes()}
// ------ /SDK

// ------ Flows
export const NangoFlows = ${JSON.stringify(parsed.integrations, null, 2)} as const;
// ------ /Flows
`;
}
export function generateInterfaces({ parsed }) {
    const interfaces = [];
    for (const [, model] of parsed.models) {
        interfaces.push(modelToTypescript({ model }));
    }
    return interfaces;
}
/**
 * Transform a JSON model to its Typescript equivalent
 */
export function modelToTypescript({ model }) {
    const output = [];
    if (model.isAnon) {
        output.push(`/** @deprecated It is recommended to use a Model */`);
        output.push(`export type ${model.name} = ${fieldToTypescript({ field: model.fields[0] })}`);
    }
    else {
        output.push(`export interface ${model.name} {`);
        output.push(...fieldsToTypescript({ fields: model.fields }));
        output.push(`};`);
    }
    return output.join('\n');
}
export function fieldsToTypescript({ fields }) {
    const output = [];
    const dynamic = fields.find((field) => field.dynamic);
    // Insert dynamic key at the beginning
    if (dynamic) {
        output.push(`  [key: string]: ${fieldToTypescript({ field: dynamic })};`);
    }
    // Regular fields
    for (const field of fields) {
        if (field.dynamic) {
            continue;
        }
        output.push(`  ${shouldQuote(field.name) ? `"${field.name}"` : field.name}${field.optional ? '?' : ''}: ${fieldToTypescript({ field: field })};`);
    }
    return output;
}
/**
 * Transform a field definition to its typescript equivalent
 */
export function fieldToTypescript({ field }) {
    if (Array.isArray(field.value)) {
        if (field.union) {
            return field.value.map((f) => fieldToTypescript({ field: f })).join(' | ');
        }
        if (field.array) {
            return `(${field.value.map((f) => fieldToTypescript({ field: f })).join(' | ')})[]`;
        }
        return `{${fieldsToTypescript({ fields: field.value }).join('\n')}}`;
    }
    if (field.model || field.tsType) {
        return `${field.value}${field.array ? '[]' : ''}`;
    }
    if (field.value === null) {
        return 'null';
    }
    if (typeof field.value === 'string') {
        return `'${field.value}${field.array ? '[]' : ''}'`;
    }
    return `${field.value}${field.array ? '[]' : ''}`;
}
/**
 * Generate SDK types
 */
export function generateSDKTypes() {
    const filePath = resolve('@nangohq/shared/dist/sdk/sync.d.ts', import.meta.url);
    const typesContent = fs.readFileSync(new URL(filePath), 'utf8');
    return `
${typesContent}

`;
}
export function generateAdditionalExports({ fullPath, parsed, debug }) {
    const exportPath = path.resolve(fullPath, '.nango');
    if (debug) {
        printDebug(`Generating schemas in ${exportPath}`);
    }
    if (!fs.existsSync(exportPath)) {
        fs.mkdirSync(exportPath, { recursive: true });
    }
    // Standalone typescript schema
    const pathTS = path.join(exportPath, 'schema.ts');
    fs.writeFileSync(pathTS, getExportToTS({ parsed }));
    if (debug) {
        printDebug(`Generated export ${pathTS}`);
    }
    // Standalone json schema
    const pathJSON = path.join(exportPath, 'schema.json');
    fs.writeFileSync(pathJSON, JSON.stringify(getExportToJSON({ pathTS }), null, 2));
    if (debug) {
        printDebug(`Generated export ${pathJSON}`);
    }
}
export function getExportToTS({ parsed }) {
    return `// ---------------------------
// This file was generated by Nango (v${NANGO_VERSION})
// You can version this file
// ---------------------------

${generateInterfaces({ parsed }).join('\n\n')}
`;
}
export function getExportToJSON({ pathTS }) {
    const schema = tsj
        .createGenerator({
        path: pathTS,
        type: '*',
        skipTypeCheck: true
    })
        .createSchema('*');
    schema.$comment = `This file was generated by Nango (v${NANGO_VERSION})`;
    // When there only 1 model it appends this useless $ref
    delete schema.$ref;
    return schema;
}
//# sourceMappingURL=model.service.js.map